# coffeelint: disable=max_line_length
'use strict'
exec = require "child_process"
  .exec
fs = require 'fs'
_ = require "lodash"
async = require 'async'
moment = require 'moment'
debug = require("../../../shared_modules/logger").debug

logger = require("../../../shared_modules/logger")

Producer = require "../../ProducerClass"

class Licenses extends Producer
  isRunning = false
  constructor: ->
    super "licenses", 15 * 1000

    dchannels = {}

    formattedTodayString = ->
      currentDate = new Date()
      year = currentDate.getFullYear()
      month = currentDate.getMonth() + 1
      month = ("0" + month).slice(-2)
      day = currentDate.getDate()
      day = ("0" + day).slice(-2)
      return "#{year}#{month}#{day}"

    outOfDateLics = []
    allLics = []

    # get all licenses information
    getAllLicenses = (noDiff) =>
      debug "getting licinfo"
      exec "licinfo -d 2>&1", (error, stdout, stderr) =>
        newstdout = {}
        if stdout
          stdout = stdout.split "\n"
          for line in stdout
            splitLine = line.split ":"
            newstdout[splitLine[0]] = splitLine[3] if splitLine[0]


          # OMG S2M
          s2m = newstdout["43MaxLines"] if newstdout["43MaxLines"]
          if not dchannels.S2M or s2m is -1 or s2m > dchannels.S2M
            s2m = dchannels.S2M
          newstdout.S2M = s2m if s2m
          delete newstdout["43MaxLines"]

          # OMG S0
          s0 = newstdout["44MaxLines"] if newstdout["44MaxLines"]
          if not dchannels.S0 or s0 is -1 or s0 > dchannels.S0
            s0 = dchannels.S0
          newstdout.S0 = s0 if s0
          delete newstdout["44MaxLines"]

          # OMG analog
          analog = newstdout["47MaxLines"] if newstdout["47MaxLines"]
          if not dchannels.FXS or analog is -1 or analog > dchannels.FXS
            analog = dchannels.FXS
          newstdout.FXS = analog if analog
          delete newstdout["47MaxLines"]

          # PCM enabled
          pcmLicensed = true if newstdout["54MaxLines"] and newstdout["54MaxLines"] isnt ("0" or "") # OMG PCM
          delete newstdout["54MaxLines"]
          # OMG PCM Lines
          pcm = newstdout["55MaxLines"] if newstdout["55MaxLines"] and pcmLicensed
          if not dchannels.PCM or pcm is -1 or pcm > dchannels.PCM
            pcm = dchannels.PCM
          newstdout.PCM = pcm if pcm

          # sip2sip Lines (= pcm lines?)
          newstdout.sip2sip = newstdout["55MaxLines"] if newstdout["55MaxLines"] and pcmLicensed
          delete newstdout["55MaxLines"]

          # OMG UM-Lines
          umLines = newstdout["45MaxLines"] if newstdout["45MaxLines"]
          # if umLines is -1 or > dchannels.UMLINES
          newstdout.umLines = umLines
          delete newstdout["45MaxLines"]

          # newstdout.lyncLines = newstdout["48MaxLines"] if newstdout["48MaxLines"] # OMG Lync-Lines
          delete newstdout["48MaxLines"]

          # IDS/OM Directory Service
          directoryService = newstdout["56Enable"] if newstdout["56Enable"]
          if directoryService is "1"
            newstdout.directoryService = "∞"
          delete newstdout["56Enable"]

          # SIP2Lync
          sip2lync = newstdout["57MaxUsers"] if newstdout["57MaxUsers"]
          if sip2lync is "-1"
            sip2lync = "∞"
          newstdout.sip2lync = sip2lync
          delete newstdout["57MaxUsers"]

          # EntryControl
          entryControl = newstdout["58MaxLines"] if newstdout["58MaxLines"]
          newstdout.entryControl = entryControl
          delete newstdout["58MaxLines"]

          newstdout.outOfDate = outOfDateLics #if outOfDateLics.length
          newstdout.all = allLics #if outOfDateLics.length
          if noDiff
            @sendData newstdout, true
          @sendData newstdout

        else if error
          logger.error error if error
          @sendError error
        else if stderr
          @sendError stderr

    # get expired licenses
    checkEachLicense = (licenseFiles) ->
      # reset outOfDateLics
      outOfDateLics = []
      allLics = []

      # after all files have been checked,
      allChecked = ->
        debug "all licenses checked. Out of date: #{outOfDateLics.length}"
        getAllLicenses "noDiff"

      checkOneLicFile = (licfile, asyncCallback) ->
        debug "checking #{licfile}"
        exec "licinfo -d -l #{licfile} -w 2>&1", { cwd: "/data/" } , (error, stdout) ->
          licObject = {}
          if stdout
            stdout = stdout.split "\n"
            for line in stdout
              splitLine = line.split ":"
              licObject[splitLine[0]] = splitLine[3] if splitLine[0]

          licExpirationDate = licObject["0DateLimit"]

          today = formattedTodayString()
          isExpired = false
          if licExpirationDate and licExpirationDate < today
            isExpired = true

          #create object with serial code and expiration date
          licenseDetails = {
            #get serial code of license
            serial: licObject["0SerialCode"]
            expirationDate: licExpirationDate
            type: if licObject["0AdditionalLicense"] and parseInt(licObject["0AdditionalLicense"]) is 1 then "extension" else "base"
            expired: isExpired
            # complete: licObject
          }
          if licExpirationDate
            licenseDetails.expirationDate = moment(licExpirationDate).format()

          outOfDateLics.push(licenseDetails) if isExpired
          allLics.push(licenseDetails)
          asyncCallback()

      #check licenseFiles in series
      async.mapSeries licenseFiles, checkOneLicFile.bind(checkOneLicFile), allChecked

    getsupportedChannels = =>

      @requestData { producer: "channels", action: "getStatus" }, (err, status, message) ->
        if message
          filteredChannels = {}
          # group bchannels by ilidType ...
          dchannelsGrouped = _.groupBy message, (chan) ->
            return chan.ilidType
          # ... and count the dchannels bchannels
          _.each dchannelsGrouped, (chanObject, chanType) ->
            filteredChannels[chanType] = _.size chanObject
          # merge
          _.merge dchannels, filteredChannels

    run = ->
      unless dchannels and not _.isEmpty dchannels
        debug "Supported channels not set, yet."
        debug JSON.stringify('dchannels', dchannels)
        getsupportedChannels()
        tryAgain = ->
          run()
        setTimeout tryAgain, 2000
        return
      fs.readdir "/data/", (err, filesInData) ->
        licenseFiles = (file for file in filesInData when file.match(/\.fle$|.flf$/) )
        checkEachLicense licenseFiles


    # run license check once on load and once a day
    getAllLicenses()
    setInterval(getAllLicenses, 24 * 60 * 60 * 1000)

    watcher = false
    watchDir = "/data/"

    @start = =>
      @debug "starting"
      @sendStatus "running"
      #run directly
      getAllLicenses()
      run()
      watcher.close() if watcher
      watcher = false
      throttledRunAfterFileChange = _.debounce(run, 100)
      watcher = fs.watch watchDir, (event, filename) ->
        throttledRunAfterFileChange() if filename.match(/\.fle|\.flf/)

    @stop = =>
      isRunning = false
      watcher.close() if watcher
      watcher = false
      @sendStatus "stopped"

  subscribe: =>
    unless isRunning
      @start()
      isRunning = true
    super()
    
module.exports = new Licenses()
